# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: departure/board/departure.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="departure/board/departure.proto",
    package="departure.proto",
    syntax="proto3",
    serialized_options=None,
    serialized_pb=_b(
        '\n\x1f\x64\x65parture/board/departure.proto\x12\x0f\x64\x65parture.proto">\n\x05Pixel\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\t\n\x01r\x18\x03 \x01(\x05\x12\t\n\x01g\x18\x04 \x01(\x05\x12\t\n\x01\x62\x18\x05 \x01(\x05"\x87\x01\n\x13\x42oardSectionContent\x12&\n\x06pixels\x18\x01 \x03(\x0b\x32\x16.departure.proto.Pixel\x12\x11\n\tcontent_w\x18\x02 \x01(\x05\x12\x11\n\tcontent_h\x18\x03 \x01(\x05\x12\x10\n\x08repeat_x\x18\x04 \x01(\x08\x12\x10\n\x08repeat_y\x18\x05 \x01(\x08":\n\nNoMovement\x12\x15\n\rx_offset_init\x18\x01 \x01(\x05\x12\x15\n\ry_offset_init\x18\x02 \x01(\x05"U\n\rStaticContent\x12\x16\n\x0etotal_duration\x18\x01 \x01(\x05\x12\x15\n\rx_offset_init\x18\x02 \x01(\x05\x12\x15\n\ry_offset_init\x18\x03 \x01(\x05"\xa0\x01\n\x10ScrollingContent\x12\x15\n\rstep_duration\x18\x01 \x01(\x05\x12\x13\n\x0btotal_steps\x18\x02 \x01(\x05\x12\x18\n\x10\x64\x65lta_x_per_step\x18\x03 \x01(\x05\x12\x18\n\x10\x64\x65lta_y_per_step\x18\x04 \x01(\x05\x12\x15\n\rx_offset_init\x18\x05 \x01(\x05\x12\x15\n\ry_offset_init\x18\x06 \x01(\x05"\xc4\x01\n\x08Movement\x12\x32\n\x0bno_movement\x18\x01 \x01(\x0b\x32\x1b.departure.proto.NoMovementH\x00\x12\x38\n\x0estatic_content\x18\x02 \x01(\x0b\x32\x1e.departure.proto.StaticContentH\x00\x12>\n\x11scrolling_content\x18\x03 \x01(\x0b\x32!.departure.proto.ScrollingContentH\x00\x42\n\n\x08movement"\xcd\x01\n\x1b\x42oardSectionOperationStatus\x12\x15\n\rsection_index\x18\x01 \x01(\x05\x12O\n\x06status\x18\x02 \x01(\x0e\x32?.departure.proto.BoardSectionOperationStatus.BoardSectionStatus"F\n\x12\x42oardSectionStatus\x12\x06\n\x02OK\x10\x00\x12\x14\n\x10SECTION_NOEXISTS\x10\x01\x12\x12\n\x0eSECTION_EXISTS\x10\x02"\xb1\x01\n\x19\x42oardSectionUpdateRequest\x12\x15\n\rsection_index\x18\x01 \x01(\x05\x12\x35\n\x07\x63ontent\x18\x02 \x01(\x0b\x32$.departure.proto.BoardSectionContent\x12+\n\x08movement\x18\x03 \x03(\x0b\x32\x19.departure.proto.Movement\x12\x19\n\x11\x63ontinue_movement\x18\x04 \x01(\x08"Z\n\x1a\x42oardSectionsUpdateRequest\x12<\n\x08requests\x18\x01 \x03(\x0b\x32*.departure.proto.BoardSectionUpdateRequest"[\n\x1b\x42oardSectionsUpdateResponse\x12<\n\x06status\x18\x01 \x03(\x0b\x32,.departure.proto.BoardSectionOperationStatus"z\n\x19\x42oardSectionCreateRequest\x12\x15\n\rsection_index\x18\x01 \x01(\x05\x12\x10\n\x08output_x\x18\x02 \x01(\x05\x12\x10\n\x08output_y\x18\x03 \x01(\x05\x12\x10\n\x08output_w\x18\x04 \x01(\x05\x12\x10\n\x08output_h\x18\x05 \x01(\x05"Z\n\x1a\x42oardSectionsCreateRequest\x12<\n\x08requests\x18\x01 \x03(\x0b\x32*.departure.proto.BoardSectionCreateRequest"[\n\x1b\x42oardSectionsCreateResponse\x12<\n\x06status\x18\x01 \x03(\x0b\x32,.departure.proto.BoardSectionOperationStatus"2\n\x19\x42oardSectionDeleteRequest\x12\x15\n\rsection_index\x18\x01 \x01(\x05"Z\n\x1a\x42oardSectionsDeleteRequest\x12<\n\x08requests\x18\x01 \x03(\x0b\x32*.departure.proto.BoardSectionDeleteRequest"[\n\x1b\x42oardSectionsDeleteResponse\x12<\n\x06status\x18\x01 \x03(\x0b\x32,.departure.proto.BoardSectionOperationStatus2\x80\x01\n\x0c\x42oardManager\x12p\n\x13\x42oardSectionsUpdate\x12+.departure.proto.BoardSectionsUpdateRequest\x1a,.departure.proto.BoardSectionsUpdateResponseb\x06proto3'
    ),
)


_BOARDSECTIONOPERATIONSTATUS_BOARDSECTIONSTATUS = _descriptor.EnumDescriptor(
    name="BoardSectionStatus",
    full_name="departure.proto.BoardSectionOperationStatus.BoardSectionStatus",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="OK", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="SECTION_NOEXISTS",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
        ),
        _descriptor.EnumValueDescriptor(
            name="SECTION_EXISTS", index=2, number=2, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=899,
    serialized_end=969,
)
_sym_db.RegisterEnumDescriptor(_BOARDSECTIONOPERATIONSTATUS_BOARDSECTIONSTATUS)


_PIXEL = _descriptor.Descriptor(
    name="Pixel",
    full_name="departure.proto.Pixel",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="x",
            full_name="departure.proto.Pixel.x",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="y",
            full_name="departure.proto.Pixel.y",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="r",
            full_name="departure.proto.Pixel.r",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="g",
            full_name="departure.proto.Pixel.g",
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="b",
            full_name="departure.proto.Pixel.b",
            index=4,
            number=5,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=52,
    serialized_end=114,
)


_BOARDSECTIONCONTENT = _descriptor.Descriptor(
    name="BoardSectionContent",
    full_name="departure.proto.BoardSectionContent",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="pixels",
            full_name="departure.proto.BoardSectionContent.pixels",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_w",
            full_name="departure.proto.BoardSectionContent.content_w",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content_h",
            full_name="departure.proto.BoardSectionContent.content_h",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="repeat_x",
            full_name="departure.proto.BoardSectionContent.repeat_x",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="repeat_y",
            full_name="departure.proto.BoardSectionContent.repeat_y",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=117,
    serialized_end=252,
)


_NOMOVEMENT = _descriptor.Descriptor(
    name="NoMovement",
    full_name="departure.proto.NoMovement",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="x_offset_init",
            full_name="departure.proto.NoMovement.x_offset_init",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="y_offset_init",
            full_name="departure.proto.NoMovement.y_offset_init",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=254,
    serialized_end=312,
)


_STATICCONTENT = _descriptor.Descriptor(
    name="StaticContent",
    full_name="departure.proto.StaticContent",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="total_duration",
            full_name="departure.proto.StaticContent.total_duration",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="x_offset_init",
            full_name="departure.proto.StaticContent.x_offset_init",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="y_offset_init",
            full_name="departure.proto.StaticContent.y_offset_init",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=314,
    serialized_end=399,
)


_SCROLLINGCONTENT = _descriptor.Descriptor(
    name="ScrollingContent",
    full_name="departure.proto.ScrollingContent",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="step_duration",
            full_name="departure.proto.ScrollingContent.step_duration",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="total_steps",
            full_name="departure.proto.ScrollingContent.total_steps",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="delta_x_per_step",
            full_name="departure.proto.ScrollingContent.delta_x_per_step",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="delta_y_per_step",
            full_name="departure.proto.ScrollingContent.delta_y_per_step",
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="x_offset_init",
            full_name="departure.proto.ScrollingContent.x_offset_init",
            index=4,
            number=5,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="y_offset_init",
            full_name="departure.proto.ScrollingContent.y_offset_init",
            index=5,
            number=6,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=402,
    serialized_end=562,
)


_MOVEMENT = _descriptor.Descriptor(
    name="Movement",
    full_name="departure.proto.Movement",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="no_movement",
            full_name="departure.proto.Movement.no_movement",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="static_content",
            full_name="departure.proto.Movement.static_content",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="scrolling_content",
            full_name="departure.proto.Movement.scrolling_content",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="movement",
            full_name="departure.proto.Movement.movement",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=565,
    serialized_end=761,
)


_BOARDSECTIONOPERATIONSTATUS = _descriptor.Descriptor(
    name="BoardSectionOperationStatus",
    full_name="departure.proto.BoardSectionOperationStatus",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="section_index",
            full_name="departure.proto.BoardSectionOperationStatus.section_index",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="status",
            full_name="departure.proto.BoardSectionOperationStatus.status",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _BOARDSECTIONOPERATIONSTATUS_BOARDSECTIONSTATUS,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=764,
    serialized_end=969,
)


_BOARDSECTIONUPDATEREQUEST = _descriptor.Descriptor(
    name="BoardSectionUpdateRequest",
    full_name="departure.proto.BoardSectionUpdateRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="section_index",
            full_name="departure.proto.BoardSectionUpdateRequest.section_index",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="content",
            full_name="departure.proto.BoardSectionUpdateRequest.content",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="movement",
            full_name="departure.proto.BoardSectionUpdateRequest.movement",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="continue_movement",
            full_name="departure.proto.BoardSectionUpdateRequest.continue_movement",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=972,
    serialized_end=1149,
)


_BOARDSECTIONSUPDATEREQUEST = _descriptor.Descriptor(
    name="BoardSectionsUpdateRequest",
    full_name="departure.proto.BoardSectionsUpdateRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="requests",
            full_name="departure.proto.BoardSectionsUpdateRequest.requests",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1151,
    serialized_end=1241,
)


_BOARDSECTIONSUPDATERESPONSE = _descriptor.Descriptor(
    name="BoardSectionsUpdateResponse",
    full_name="departure.proto.BoardSectionsUpdateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="status",
            full_name="departure.proto.BoardSectionsUpdateResponse.status",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1243,
    serialized_end=1334,
)


_BOARDSECTIONCREATEREQUEST = _descriptor.Descriptor(
    name="BoardSectionCreateRequest",
    full_name="departure.proto.BoardSectionCreateRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="section_index",
            full_name="departure.proto.BoardSectionCreateRequest.section_index",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="output_x",
            full_name="departure.proto.BoardSectionCreateRequest.output_x",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="output_y",
            full_name="departure.proto.BoardSectionCreateRequest.output_y",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="output_w",
            full_name="departure.proto.BoardSectionCreateRequest.output_w",
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="output_h",
            full_name="departure.proto.BoardSectionCreateRequest.output_h",
            index=4,
            number=5,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1336,
    serialized_end=1458,
)


_BOARDSECTIONSCREATEREQUEST = _descriptor.Descriptor(
    name="BoardSectionsCreateRequest",
    full_name="departure.proto.BoardSectionsCreateRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="requests",
            full_name="departure.proto.BoardSectionsCreateRequest.requests",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1460,
    serialized_end=1550,
)


_BOARDSECTIONSCREATERESPONSE = _descriptor.Descriptor(
    name="BoardSectionsCreateResponse",
    full_name="departure.proto.BoardSectionsCreateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="status",
            full_name="departure.proto.BoardSectionsCreateResponse.status",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1552,
    serialized_end=1643,
)


_BOARDSECTIONDELETEREQUEST = _descriptor.Descriptor(
    name="BoardSectionDeleteRequest",
    full_name="departure.proto.BoardSectionDeleteRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="section_index",
            full_name="departure.proto.BoardSectionDeleteRequest.section_index",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1645,
    serialized_end=1695,
)


_BOARDSECTIONSDELETEREQUEST = _descriptor.Descriptor(
    name="BoardSectionsDeleteRequest",
    full_name="departure.proto.BoardSectionsDeleteRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="requests",
            full_name="departure.proto.BoardSectionsDeleteRequest.requests",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1697,
    serialized_end=1787,
)


_BOARDSECTIONSDELETERESPONSE = _descriptor.Descriptor(
    name="BoardSectionsDeleteResponse",
    full_name="departure.proto.BoardSectionsDeleteResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="status",
            full_name="departure.proto.BoardSectionsDeleteResponse.status",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1789,
    serialized_end=1880,
)

_BOARDSECTIONCONTENT.fields_by_name["pixels"].message_type = _PIXEL
_MOVEMENT.fields_by_name["no_movement"].message_type = _NOMOVEMENT
_MOVEMENT.fields_by_name["static_content"].message_type = _STATICCONTENT
_MOVEMENT.fields_by_name["scrolling_content"].message_type = _SCROLLINGCONTENT
_MOVEMENT.oneofs_by_name["movement"].fields.append(
    _MOVEMENT.fields_by_name["no_movement"]
)
_MOVEMENT.fields_by_name["no_movement"].containing_oneof = _MOVEMENT.oneofs_by_name[
    "movement"
]
_MOVEMENT.oneofs_by_name["movement"].fields.append(
    _MOVEMENT.fields_by_name["static_content"]
)
_MOVEMENT.fields_by_name["static_content"].containing_oneof = _MOVEMENT.oneofs_by_name[
    "movement"
]
_MOVEMENT.oneofs_by_name["movement"].fields.append(
    _MOVEMENT.fields_by_name["scrolling_content"]
)
_MOVEMENT.fields_by_name[
    "scrolling_content"
].containing_oneof = _MOVEMENT.oneofs_by_name["movement"]
_BOARDSECTIONOPERATIONSTATUS.fields_by_name[
    "status"
].enum_type = _BOARDSECTIONOPERATIONSTATUS_BOARDSECTIONSTATUS
_BOARDSECTIONOPERATIONSTATUS_BOARDSECTIONSTATUS.containing_type = (
    _BOARDSECTIONOPERATIONSTATUS
)
_BOARDSECTIONUPDATEREQUEST.fields_by_name["content"].message_type = _BOARDSECTIONCONTENT
_BOARDSECTIONUPDATEREQUEST.fields_by_name["movement"].message_type = _MOVEMENT
_BOARDSECTIONSUPDATEREQUEST.fields_by_name[
    "requests"
].message_type = _BOARDSECTIONUPDATEREQUEST
_BOARDSECTIONSUPDATERESPONSE.fields_by_name[
    "status"
].message_type = _BOARDSECTIONOPERATIONSTATUS
_BOARDSECTIONSCREATEREQUEST.fields_by_name[
    "requests"
].message_type = _BOARDSECTIONCREATEREQUEST
_BOARDSECTIONSCREATERESPONSE.fields_by_name[
    "status"
].message_type = _BOARDSECTIONOPERATIONSTATUS
_BOARDSECTIONSDELETEREQUEST.fields_by_name[
    "requests"
].message_type = _BOARDSECTIONDELETEREQUEST
_BOARDSECTIONSDELETERESPONSE.fields_by_name[
    "status"
].message_type = _BOARDSECTIONOPERATIONSTATUS
DESCRIPTOR.message_types_by_name["Pixel"] = _PIXEL
DESCRIPTOR.message_types_by_name["BoardSectionContent"] = _BOARDSECTIONCONTENT
DESCRIPTOR.message_types_by_name["NoMovement"] = _NOMOVEMENT
DESCRIPTOR.message_types_by_name["StaticContent"] = _STATICCONTENT
DESCRIPTOR.message_types_by_name["ScrollingContent"] = _SCROLLINGCONTENT
DESCRIPTOR.message_types_by_name["Movement"] = _MOVEMENT
DESCRIPTOR.message_types_by_name[
    "BoardSectionOperationStatus"
] = _BOARDSECTIONOPERATIONSTATUS
DESCRIPTOR.message_types_by_name[
    "BoardSectionUpdateRequest"
] = _BOARDSECTIONUPDATEREQUEST
DESCRIPTOR.message_types_by_name[
    "BoardSectionsUpdateRequest"
] = _BOARDSECTIONSUPDATEREQUEST
DESCRIPTOR.message_types_by_name[
    "BoardSectionsUpdateResponse"
] = _BOARDSECTIONSUPDATERESPONSE
DESCRIPTOR.message_types_by_name[
    "BoardSectionCreateRequest"
] = _BOARDSECTIONCREATEREQUEST
DESCRIPTOR.message_types_by_name[
    "BoardSectionsCreateRequest"
] = _BOARDSECTIONSCREATEREQUEST
DESCRIPTOR.message_types_by_name[
    "BoardSectionsCreateResponse"
] = _BOARDSECTIONSCREATERESPONSE
DESCRIPTOR.message_types_by_name[
    "BoardSectionDeleteRequest"
] = _BOARDSECTIONDELETEREQUEST
DESCRIPTOR.message_types_by_name[
    "BoardSectionsDeleteRequest"
] = _BOARDSECTIONSDELETEREQUEST
DESCRIPTOR.message_types_by_name[
    "BoardSectionsDeleteResponse"
] = _BOARDSECTIONSDELETERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Pixel = _reflection.GeneratedProtocolMessageType(
    "Pixel",
    (_message.Message,),
    dict(
        DESCRIPTOR=_PIXEL,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.Pixel)
    ),
)
_sym_db.RegisterMessage(Pixel)

BoardSectionContent = _reflection.GeneratedProtocolMessageType(
    "BoardSectionContent",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONCONTENT,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionContent)
    ),
)
_sym_db.RegisterMessage(BoardSectionContent)

NoMovement = _reflection.GeneratedProtocolMessageType(
    "NoMovement",
    (_message.Message,),
    dict(
        DESCRIPTOR=_NOMOVEMENT,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.NoMovement)
    ),
)
_sym_db.RegisterMessage(NoMovement)

StaticContent = _reflection.GeneratedProtocolMessageType(
    "StaticContent",
    (_message.Message,),
    dict(
        DESCRIPTOR=_STATICCONTENT,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.StaticContent)
    ),
)
_sym_db.RegisterMessage(StaticContent)

ScrollingContent = _reflection.GeneratedProtocolMessageType(
    "ScrollingContent",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SCROLLINGCONTENT,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.ScrollingContent)
    ),
)
_sym_db.RegisterMessage(ScrollingContent)

Movement = _reflection.GeneratedProtocolMessageType(
    "Movement",
    (_message.Message,),
    dict(
        DESCRIPTOR=_MOVEMENT,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.Movement)
    ),
)
_sym_db.RegisterMessage(Movement)

BoardSectionOperationStatus = _reflection.GeneratedProtocolMessageType(
    "BoardSectionOperationStatus",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONOPERATIONSTATUS,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionOperationStatus)
    ),
)
_sym_db.RegisterMessage(BoardSectionOperationStatus)

BoardSectionUpdateRequest = _reflection.GeneratedProtocolMessageType(
    "BoardSectionUpdateRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONUPDATEREQUEST,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionUpdateRequest)
    ),
)
_sym_db.RegisterMessage(BoardSectionUpdateRequest)

BoardSectionsUpdateRequest = _reflection.GeneratedProtocolMessageType(
    "BoardSectionsUpdateRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONSUPDATEREQUEST,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionsUpdateRequest)
    ),
)
_sym_db.RegisterMessage(BoardSectionsUpdateRequest)

BoardSectionsUpdateResponse = _reflection.GeneratedProtocolMessageType(
    "BoardSectionsUpdateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONSUPDATERESPONSE,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionsUpdateResponse)
    ),
)
_sym_db.RegisterMessage(BoardSectionsUpdateResponse)

BoardSectionCreateRequest = _reflection.GeneratedProtocolMessageType(
    "BoardSectionCreateRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONCREATEREQUEST,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionCreateRequest)
    ),
)
_sym_db.RegisterMessage(BoardSectionCreateRequest)

BoardSectionsCreateRequest = _reflection.GeneratedProtocolMessageType(
    "BoardSectionsCreateRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONSCREATEREQUEST,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionsCreateRequest)
    ),
)
_sym_db.RegisterMessage(BoardSectionsCreateRequest)

BoardSectionsCreateResponse = _reflection.GeneratedProtocolMessageType(
    "BoardSectionsCreateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONSCREATERESPONSE,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionsCreateResponse)
    ),
)
_sym_db.RegisterMessage(BoardSectionsCreateResponse)

BoardSectionDeleteRequest = _reflection.GeneratedProtocolMessageType(
    "BoardSectionDeleteRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONDELETEREQUEST,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionDeleteRequest)
    ),
)
_sym_db.RegisterMessage(BoardSectionDeleteRequest)

BoardSectionsDeleteRequest = _reflection.GeneratedProtocolMessageType(
    "BoardSectionsDeleteRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONSDELETEREQUEST,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionsDeleteRequest)
    ),
)
_sym_db.RegisterMessage(BoardSectionsDeleteRequest)

BoardSectionsDeleteResponse = _reflection.GeneratedProtocolMessageType(
    "BoardSectionsDeleteResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOARDSECTIONSDELETERESPONSE,
        __module__="departure.board.departure_pb2"
        # @@protoc_insertion_point(class_scope:departure.proto.BoardSectionsDeleteResponse)
    ),
)
_sym_db.RegisterMessage(BoardSectionsDeleteResponse)


_BOARDMANAGER = _descriptor.ServiceDescriptor(
    name="BoardManager",
    full_name="departure.proto.BoardManager",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    serialized_start=1883,
    serialized_end=2011,
    methods=[
        _descriptor.MethodDescriptor(
            name="BoardSectionsUpdate",
            full_name="departure.proto.BoardManager.BoardSectionsUpdate",
            index=0,
            containing_service=None,
            input_type=_BOARDSECTIONSUPDATEREQUEST,
            output_type=_BOARDSECTIONSUPDATERESPONSE,
            serialized_options=None,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_BOARDMANAGER)

DESCRIPTOR.services_by_name["BoardManager"] = _BOARDMANAGER

# @@protoc_insertion_point(module_scope)
